package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class CBCBlockCipher implements BlockCipher {
   private byte[] IV;
   private byte[] cbcV;
   private byte[] cbcNextV;
   private int blockSize;
   private BlockCipher cipher;
   private boolean encrypting;

   public CBCBlockCipher(BlockCipher cipher) {
      CBCBlockCipher var10000 = this;
      CBCBlockCipher var10001 = this;
      CBCBlockCipher var10002 = this;
      super();
      this.cipher = null;
      this.cipher = cipher;
      int var2;
      this.blockSize = var2 = cipher.getBlockSize();
      var10002.IV = new byte[var2];
      var10001.cbcV = new byte[var2];
      var10000.cbcNextV = new byte[var2];
   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (inOff + this.blockSize > in.length) {
         throw new DataLengthException("input buffer too short");
      } else {
         for(int var5 = 0; var5 < this.blockSize; ++var5) {
            byte[] var10000 = this.cbcV;
            var10000[var5] ^= in[inOff + var5];
         }

         int var8 = this.cipher.processBlock(this.cbcV, 0, out, outOff);
         byte[] var6;
         int in1 = (var6 = this.cbcV).length;
         System.arraycopy(out, outOff, var6, 0, in1);
         return var8;
      }
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else {
         System.arraycopy(in, inOff, this.cbcNextV, 0, var5);
         int in1 = this.cipher.processBlock(in, inOff, out, outOff);

         for(inOff = 0; inOff < this.blockSize; ++inOff) {
            out[var5] = (byte)(out[var5 = outOff + inOff] ^ this.cbcV[inOff]);
         }

         byte[] var6 = this.cbcV;
         this.cbcV = this.cbcNextV;
         this.cbcNextV = var6;
         return in1;
      }
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher;
   }

   public void init(boolean encrypting, CipherParameters params) {
      this.encrypting = encrypting;
      if (params instanceof ParametersWithIV) {
         byte[] var3;
         ParametersWithIV params1;
         if ((var3 = (params1 = (ParametersWithIV)params).getIV()).length != this.blockSize) {
            throw new IllegalArgumentException("initialisation vector must be the same length as block size");
         }

         CBCBlockCipher var10000 = this;
         CBCBlockCipher var10001 = this;
         byte[] var10002 = var3;
         byte[] var4 = this.IV;
         int var6 = var3.length;
         System.arraycopy(var10002, 0, var4, 0, var6);
         var10001.reset();
         var10000.cipher.init(encrypting, params1.getParameters());
      } else {
         this.reset();
         this.cipher.init(encrypting, params);
      }

   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName() + "/CBC";
   }

   public int getBlockSize() {
      return this.cipher.getBlockSize();
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      return this.encrypting ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
   }

   public void reset() {
      CBCBlockCipher var10000 = this;
      byte[] var10001 = this.IV;
      byte[] var2 = this.cbcV;
      int var1 = var10001.length;
      System.arraycopy(var10001, 0, var2, 0, var1);
      var10000.cipher.reset();
   }
}
